<table class="table">
    <thead>
        <tr>
            <th scope="col">module</th>
            <th scope="col">class</th>
            <th scope="col">test</th>
            <th scope="col">status</th>
        <tr>
    </thead>
    <tbody>
    {% for  result in result_list -%}
        <tr>
            <td>{{result.module_name}}</td>
            <td>{{result.class_name}}</td>
            <td>{{result.test_name}}</td>
            <td>{{result.status}}</td>
        </tr>
    {% endfor %}</tbody>
</table>